# Generated by Django 4.0.2 on 2022-02-18 11:28

from django.contrib.postgres.operations import TrigramExtension
from django.db import migrations


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        TrigramExtension(),  # sets up the extension in PostgreSQL
    ]
